/**
  *  \file arch/win32-bcc55/platform.cc
  */

#include <windows.h>
#include "arch/win32-bcc55/platform.h"
#include "io/dirs.h"

static string_t
getProgDirName()
{
    char buffer[1024];
    int len = GetModuleFileName(NULL, buffer, sizeof(buffer));
    return Directory::getDirName(string_t(buffer, len));
}

string_t
Platform::getDataDirectory()
{
    return Directory::makeFileName(getProgDirName(), "specs");
}

string_t
Platform::getResourceDirectory()
{
    return Directory::makeFileName(getProgDirName(), "resource");
}

string_t
Platform::getLanguageCode()
{
    /* According to Win32.hlp:
       # LOCALE_SABBREVLANGNAME:
       #    Abbreviated name of the language, created by taking the
       #    2-letter language abbreviation from the ISO Standard 639 and
       #    adding a third letter, as appropriate, to indicate the
       #    sublanguage.
       So, when we get `enu', we make `en_u' from it. Under Unix,
       the relevant code would be `en_US', but we only use the part
       before the underscore anyway. */

    /* "Language codes and scripts":
      Language/Locale           Code    Script        Unix equivalent
      Greek                     ELL     Other         el_GR.iso8859-7
      Russian                   RUS     Cyrillic      ru_RU
      Turkish                   TRK     Latin 2       tr_TR
      Polish                    PLK     Latin 2       pl_PL
      Czech                     CSY     Latin 2       cs_CZ
      Slovak                    SKY     Latin 2       sk_SK
      Hungarian                 HUN     Latin 2       hu_HU
      Danish                    DAN     Latin 1       da_DK
      Dutch (Standard)          NLD     Latin 1       nl (??)
      Belgian (Flemish)         NLB     Latin 1       be (??)
      English (American)        ENU     Latin 1       en_US
      English (British)         ENG     Latin 1       en_GB
      English (Australian)      ENA     Latin 1
      English (Canadian)        ENC     Latin 1
      English (New Zealand)     ENZ     Latin 1
      English (Ireland)         ENI     Latin 1
      Finnish                   FIN     Latin 1       fi_FI
      French (Standard)         FRA     Latin 1       fr_FR
      French (Belgian)          FRB     Latin 1
      French (Canadian)         FRC     Latin 1
      French (Swiss)            FRS     Latin 1
      German (Standard)         DEU     Latin 1       de_DE
      German (Swiss)            DES     Latin 1       de_CH
      German (Austrian)         DEA     Latin 1       de_AT
      Icelandic                 ISL     Latin 1       is_IS
      Italian (Standard)        ITA     Latin 1       it_IT
      Italian (Swiss)           ITS     Latin 1
      Norwegian (Bokmal)        NOR     Latin 1       no_NO
      Norwegian (Nynorsk)       NON     Latin 1       nn_NO
      Portuguese (Brazilian)    PTB     Latin 1       pt_BR
      Portuguese (Standard)     PTG     Latin 1       pt_PT
      Swedish                   SVE     Latin 1       sv_SE
      Spanish (Std/Trad)        ESP     Latin 1       es_ES
      Spanish (Mexican)         ESM     Latin 1
      Spanish (Modern)          ESN     Latin 1 */
    
    char buffer[10];
    int n = GetLocaleInfo(LOCALE_USER_DEFAULT, LOCALE_SABBREVLANGNAME,
                          buffer, sizeof buffer);
    if (n <= 1) {
        return string_t();
    } else {
        --n;
        if (n <= 2)
            return string_t(buffer, n);
        else
            return string_t(buffer, 2) + "_" + string_t(buffer+2, n-2);
    }
}
