/**
  *  \file int/stmt.h
  *
  *  (c) 2001-2007 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_INT_STMT_H
#define PCC_V2_INT_STMT_H

#include "int/lex.h"
#include "cpluslib/string.h"
#include "int/bytecode.h"

class IntCommandSource;

class IntStatementCompiler {
    IntLexer            lex;
    IntCommandSource&   source;
    IntByteCodeObject&  bco;

    IntByteCodeObject::label_t  break_addr, continue_addr;
    int                         with_level, context_level;

    void compileError(const char* text);
    void compileError(const string_t& s);

    void nextLine();
    bool parseStatementsUntil(const char* term1, const char* term2);
    void parseExpression();

    void parseDoBody(IntByteCodeObject::label_t again);
    void parseForEach(bool oneline, string_t content);
    void parseWith(bool oneline, string_t content);
    void parseDim(IntOpcode op);
    IntByteCodeObject& parseSub();
 public:
    IntStatementCompiler(IntCommandSource& source, IntByteCodeObject& bco,
                         int context_level,
                         IntByteCodeObject::label_t break_addr = IntByteCodeObject::NO_LABEL,
                         IntByteCodeObject::label_t continue_addr = IntByteCodeObject::NO_LABEL,
                         int with_level = 0);
    ~IntStatementCompiler();

    void parseStatement(bool allow_multiline);
    bool parseWholeStatement(bool allow_multiline);
};

#endif
