/**
  *  \file tests/testcard.cc
  */

#include <SDL/SDL.h>
#include "ui/widgets.h"
#include "gfx/palette.h"
#include "ui/window.h"
#include "ui/layout.h"
#include "cpluslib/holder.h"
#include "util/debug.h"
#include "gfx/gfx.h"

class TestWindow : public UIWindow {
    UICardGroup& cg;
 public:
    TestWindow(UICardGroup& cg)
        : UIWindow("Window", UIWindow::BLUE_WINDOW, COLOR_DARK), cg(cg) { }
    bool handleEvent(const UIEvent& e, bool second_pass);
};

bool
TestWindow::handleEvent(const UIEvent& e, bool second_pass)
{
    if (e.type == ev_Command) {
        switch(e.cmd.code) {
         case 1:
         case 2:
         case 3:
         case 4:
            std::cout << "Button " << e.cmd.code << " pressed\n";
            return true;
         case 99: {
            UIWidget* w = cg.getFocusedWidget();
            if (w && w->getId() < 4)
                cg.getWidgetById(w->getId()+1)->setFocus();
            return true;
         }
         case 100:
            stop(1);
            return true;
        }
    }
    return UIWindow::handleEvent(e, second_pass);
}

int
main(int /*argc*/, char** /*argv*/)
{
    if (SDL_Init(SDL_INIT_VIDEO) != 0) {
        printf("error initializing SDL: %s\n", SDL_GetError());
        return 1;
    }
    atexit(SDL_Quit);

    SDL_Surface* sfc = SDL_SetVideoMode(640, 480, 16, SDL_HWPALETTE);
    if (!sfc) {
        printf("can't set video mode\n");
        return 1;
    }

    enableStdioConsole();
    initDebug();

    initPalette(sfc);
    initUIGlobals(new GfxSurface(sfc, false),
                  Directory::openDir("resource"));
//     initUIGlobals(new GfxNullCanvas(),
//                   Directory::openDir("resource"));

    ObjectHolder h;
    UICardGroup cg;
    TestWindow win(cg);
    win.setLayout(new UIVBoxLayout(5));
    UICardGroupTab tab(cg);
    tab.addTab(1, "one", 'a');
    tab.addTab(2, "two", 'b');
    tab.addTab(3, "thr", 'c');
    tab.addTab(4, "fou", 'd');
    win.add(tab);
    cg.add(h.add(new UIButton(1, "ButtonOne", '1')));
    cg.add(h.add(new UIButton(2, "ButtonTwo", '2')));
    cg.add(h.add(new UIButton(3, "ButtonThree", '3')));
    cg.add(h.add(new UIButton(4, "ButtonFour", '4')));
    win.add(cg);
    win.add(h.add(new UIButton(99, "Next", 'n')));
    win.add(h.add(new UIButton(100, "Exit", SDLK_ESCAPE)));

    win.pack();

    ui_root->centerWidget(win);
    ui_root->add(win);
    ui_root->run();
}
