/**
  *  \file ui/widget-busy.cc
  *
  *  (c) 2001-2007 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "ui/widget-busy.h"
#include "ui/window.h"
#include "util/nls.h"
#include "gfx/palette.h"
#include "ui/uiglobal.h"
#include "ui/util.h"
#include "ui/event.h"

/** Constructor. */
UIBusyWidget::UIBusyWidget()
    : UIFixedWidget(0),
      phase(0), last_time(0)
{
    int width = fonts[FONT_TITLE]->getTextWidth(_("Wait..."));
    int height = font_heights[FONT_TITLE];

    setExtent(GfxRect(0, 0, width + 16, height + 8));
    ui_root->centerWidget(*this);
}

/** Destructor. */
UIBusyWidget::~UIBusyWidget()
{ }

/** Draw the widget's frame. */
void
UIBusyWidget::drawFrame(GfxCanvas& can)
{
    uint8 p = phase & 7;

    int x0 = getExtent().x + 3;
    int x1 = getExtent().x + getExtent().w - 4;
    int y0 = getExtent().y + 3;
    int y1 = getExtent().y + getExtent().h - 4;

    uint32 color1 = standard_colors[COLOR_YELLOW];
    uint32 color0 = standard_colors[COLOR_DARK];

    int x = x0, y = y0;
    /* top-left to top-right */
    while (x < x1) {
        can.drawPixel(x, y, ((p & 7) < 4) ? color0 : color1, SDL_ALPHA_OPAQUE);
        ++p;
        ++x;
    }
    /* top-right to bottom-right */
    while (y < y1) {
        can.drawPixel(x, y, ((p & 7) < 4) ? color0 : color1, SDL_ALPHA_OPAQUE);
        ++p;
        ++y;
    }
    /* bottom-right to bottom-left */
    while (x > x0) {
        can.drawPixel(x, y, ((p & 7) < 4) ? color0 : color1, SDL_ALPHA_OPAQUE);
        ++p;
        --x;
    }
    /* bottom-left to top-left */
    while (y > y0) {
        can.drawPixel(x, y, ((p & 7) < 4) ? color0 : color1, SDL_ALPHA_OPAQUE);
        ++p;
        --y;
    }
}

void
UIBusyWidget::draw(GfxCanvas& can, UpdateInfo& ui)
{
    GfxRect r = getExtent();
    drawFrameUp(can, r);
    r.grow(-1, -1);
    drawSolidBar(can, r, standard_colors[COLOR_DARK]); // FIXME: skin?

    GfxParameters parm;
    parm.color = standard_colors[COLOR_WHITE];
    parm.just_x = parm.just_y = 1;
    parm.font = fonts[FONT_TITLE];
    outText(can, parm, r.x + r.w/2, r.y + r.h/2, _("Wait..."));

    drawFrame(can);

    ui.update(getExtent());
}

/** Advance time. This causes the yellow frame to advance to indicate
    activity. We limit movement to a rate of 20 Hz, to make it more
    visually appealing. */
void
UIBusyWidget::tick()
{
    long now = getTimer();
    if (now > last_time + TIMER_HZ/20) {
        last_time = now;
        --phase;
        if (UIWidgetContainer* g = getOwner()) {
            ObjectHolder h;
            GfxCanvas& can = g->getCanvasFor(this, h);
            drawFrame(can);
            can.update(getExtent());
        }
    }
}
